// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef FRAMEWORKINTEGRATION_VERSION_H
#define FRAMEWORKINTEGRATION_VERSION_H

#define FRAMEWORKINTEGRATION_VERSION_STRING "6.18.0"
#define FRAMEWORKINTEGRATION_VERSION_MAJOR 6
#define FRAMEWORKINTEGRATION_VERSION_MINOR 18
#define FRAMEWORKINTEGRATION_VERSION_PATCH 0
#define FRAMEWORKINTEGRATION_VERSION ((6<<16)|(18<<8)|(0))

#endif
